//
//  ActivityIndicator.m
//  RNMedusaUI
//
//  Created by Vivien Bartha on 07/10/2020.
//  Copyright © 2020 Bosch. All rights reserved.
//

#import "React/UIView+React.h"
#import <React/RCTConvert.h>

#import "ActivityIndicator.h"

// For the Bosch indicator the cycle lasts for 3s and there are 8 transitions
const CGFloat MOVEMENT_DURATION = 3.0 / 8;

@implementation ActivityIndicator

- (dispatch_queue_t)methodQueue
{
    return dispatch_get_main_queue();
}

- (instancetype) init
{
    self = [super init];
    return [self initialize];
}

// Update view size and indicator position when size changes on JS side
- (void)reactSetFrame:(CGRect)frame {
    [super reactSetFrame: frame];

    CGRect wrapperFrame = _wrapperView.frame;
    wrapperFrame.origin.x = (frame.size.width - _wrapperSize) / 2;
    [_wrapperView setFrame:wrapperFrame];
}

// Handle receiving boxSize prop from JS
- (void) setBoxSize: (CGFloat)boxSize
{
    _boxSize = boxSize;
    [self initialize];
}

// Handle receiving topColors prop from JS
- (void) setTopColors: (NSArray *)topColorStrings
{
    _topColors = [self convertColors: topColorStrings];
}

// Handle receiving bottomColors prop from JS
- (void) setBottomColors: (NSArray *)bottomColorStrings
{
    _bottomColors = [self convertColors: bottomColorStrings];
}

// Handle receiving middleColors prop from JS
- (void) setMiddleColors: (NSArray *)middleColorStrings
{
    _middleColors = [self convertColors: middleColorStrings];
}

// Conversion logic taken from react-native-linear-gradient
- (NSArray *)convertColors: (NSArray *)colorStrings
{
    NSMutableArray *colors = [NSMutableArray arrayWithCapacity:colorStrings.count];
    for (NSString *colorString in colorStrings)
    {
        if ([colorString isKindOfClass:UIColor.class])
        {
            [colors addObject:(UIColor *)colorString];
        }
        else
        {
            [colors addObject:[RCTConvert UIColor:colorString]];
        }
    }

    return colors;
}

- (instancetype) initialize
{
    [self setUserInteractionEnabled: NO];

    _animating = NO;
    // These will be the steps of the animation and the steps will start from the beginning when reaching the end
    _animationFns = [NSArray arrayWithObjects: [NSValue valueWithPointer: @selector(animateHorizontalOut:)],
                     [NSValue valueWithPointer: @selector(animateHorizontalIn:)],
                     [NSValue valueWithPointer: @selector(animateVerticalOut:)],
                     [NSValue valueWithPointer: @selector(animateVerticalIn:)],
                     nil
                     ];

    _wrapperSize = _boxSize * 3;
    _wrapperView = [[UIView alloc] initWithFrame: CGRectMake((self.bounds.size.width  - _wrapperSize) / 2,
                                                             0,
                                                             _boxSize * 3,
                                                             _boxSize * 3)];
    _wrapperView.opaque = NO;
    _wrapperView.backgroundColor = UIColor.clearColor;
    [self addSubview: _wrapperView];

    CGRect frame = CGRectMake((_wrapperSize  - _boxSize) / 2,
                              (_wrapperSize - _boxSize) / 2,
                              _boxSize, _boxSize);

    _topLeftView = [[UIView alloc] initWithFrame: frame];
    [_wrapperView addSubview: _topLeftView];

    _bottomRightView = [[UIView alloc] initWithFrame: frame];
    [_wrapperView addSubview: _bottomRightView];

    _middleView = [[UIView alloc] initWithFrame:frame];
    [_wrapperView addSubview: _middleView];

    _maskView = [[UIView alloc] initWithFrame:frame];
    _maskView.backgroundColor = UIColor.blackColor;
    _middleView.layer.mask = _maskView.layer;

    return self;
}

- (void)startAnimation
{
    if (!_animating) {
        _animating =  YES;

        dispatch_async(dispatch_get_main_queue(), ^(void) {

            [self resetSubViews];

            [self runAnimation: 0];

        });
    }
}

- (void)restartAnimation
{
    if (_animating) {
        // If animations were interrupted (e.g. by modal closing), but still should be animating
        // then restart the animations

        dispatch_async(dispatch_get_main_queue(), ^(void) {

            [self resetSubViews];
            [self runAnimation: 0];

        });
    }
}

- (void)stopAnimation
{
    dispatch_async(dispatch_get_main_queue(), ^(void) {
        _animating = NO;

        [UIView animateWithDuration: 0.3
                         animations: ^{

            self->_topLeftView.alpha  = 0.0;
            self->_bottomRightView.alpha = 0.0;
            self->_middleView.alpha  = 0.0;

            CGPoint center = CGPointMake(
                                         self->_wrapperView.bounds.size.width / 2,
                                         self->_wrapperView.bounds.size.height / 2
                                         );

            self->_topLeftView.center  = center;
            self->_bottomRightView.center = center;
            self->_middleView.center  = center;
        }
                         completion: ^(BOOL finished) {

            [self removeAllAnimations];

        }];
    });
}

// Cycles the animations defined in _animationFns
- (void) runAnimation: (long) animationIndex
{
    if (_animating) {
        [self performSelector: [_animationFns[animationIndex] pointerValue] withObject: ^{
            [self runAnimation: (animationIndex + 1) % self->_animationFns.count];
        }];
    }
}

// ============ Animation steps ============

- (void) animateVerticalOut: (void (^)(void))next
{
    [UIView animateWithDuration: MOVEMENT_DURATION
                          delay: 0.0
                        options: UIViewAnimationOptionCurveEaseOut
                     animations: ^{

        [self moveOutwards: YES];

    }
                     completion: ^(BOOL finished) {

        if (finished) {
            self->_middleColorIndex = (self->_middleColorIndex + 1) % self->_middleColors.count;
            [self updateColors];

            next();
        } else {
            [self restartAnimation];
        }

    }];
}

- (void) animateVerticalIn: (void (^)(void))next
{
    [UIView animateWithDuration: MOVEMENT_DURATION
                          delay: 0.0
                        options: UIViewAnimationOptionCurveEaseIn
                     animations: ^{

        [self moveInwards];

    }
                     completion: ^(BOOL finished) {

        if (finished) {
            self->_bottomColorIndex = (self->_bottomColorIndex + 1) % self->_bottomColors.count;
            [self updateColors];

            next();
        } else {
            [self restartAnimation];
        }

    }];
}

- (void) animateHorizontalOut: (void (^)(void))next
{
    [UIView animateWithDuration: MOVEMENT_DURATION
                          delay: 0.0
                        options: UIViewAnimationOptionCurveEaseOut
                     animations: ^{

        [self moveOutwards: NO];

    }
                     completion: ^(BOOL finished) {

        if (finished) {
            self->_middleColorIndex = (self->_middleColorIndex + 1) % self->_middleColors.count;
            [self updateColors];

            next();
        } else {
            [self restartAnimation];
        }

    }];
}

- (void) animateHorizontalIn: (void (^)(void))next
{
    [UIView animateWithDuration: MOVEMENT_DURATION
                          delay: 0.0
                        options: UIViewAnimationOptionCurveEaseIn
                     animations: ^{

        [self moveInwards];

    }
                     completion: ^(BOOL finished) {

        if (finished) {
            self->_topColorIndex = (self->_topColorIndex + 1) % self->_topColors.count;
            [self updateColors];

            next();
        } else {
            [self restartAnimation];
        }

    }];
}

// ============ UI changes ============

- (void) moveOutwards: (bool) vertical
{
    CGPoint center = CGPointMake(_wrapperView.bounds.size.width / 2, _wrapperView.bounds.size.height / 2);
    CGSize offset = vertical ? CGSizeMake(0, _boxSize) : CGSizeMake(_boxSize, 0);

    _topLeftView.center  = CGPointMake(center.x - offset.width, center.y - offset.height);
    _bottomRightView.center = CGPointMake(center.x + offset.width, center.y + offset.height);
    _middleView.center  = CGPointMake(center.x + offset.width, center.y + offset.height);

    _maskView.center = vertical ? CGPointMake(_boxSize / 2, -_boxSize * 1.55) : CGPointMake(-_boxSize * 1.55, _boxSize / 2);
}

- (void) moveInwards
{
    CGPoint center = CGPointMake(_wrapperView.bounds.size.width / 2, _wrapperView.bounds.size.height / 2);

    _topLeftView.center  = center;
    _bottomRightView.center = center;
    _middleView.center  = center;

    _maskView.center = CGPointMake(_boxSize / 2, _boxSize / 2);

}

- (void) updateColors
{
    _bottomRightView.backgroundColor = _bottomColors[_bottomColorIndex];
    _middleView.backgroundColor = _middleColors[_middleColorIndex];
    _topLeftView.backgroundColor = _topColors[_topColorIndex];
}

// ============ Cleanup ============

- (void) resetSubViews
{
    _topColorIndex = 0;
    _bottomColorIndex = 0;
    _middleColorIndex = 0;

    [self updateColors];

    CGPoint center = CGPointMake(_wrapperView.bounds.size.width / 2, _wrapperView.bounds.size.height / 2);

    _topLeftView.center = center;
    _bottomRightView.center = center;
    _middleView.center = center;

    _maskView.center = CGPointMake(_boxSize / 2, _boxSize / 2);
}

- (void) removeAllAnimations
{
    [self.layer removeAllAnimations];
    [_topLeftView.layer removeAllAnimations];
    [_bottomRightView.layer removeAllAnimations];
    [_middleView.layer removeAllAnimations];
    [_maskView.layer removeAllAnimations];
}

@end
